function [thr,pp,bb,y_av,dirac,neg] = inputs_check(data)

if size(data,2)==3
    data = [data NaN(size(data,1),1)];
end
neg = NaN;
dirac = NaN;
   
%% 1. Average income/wealth
    [y_av]= data(1,2); %average income
    if ~isscalar(y_av)||~isfloat(y_av)||~(y_av>0)
        error('Average income should be a positive real number.')
    end
    data = data(3:size(data,1),:);

%% 2. All thresholds are nonnegative

if data(1,1)>=0
    if (data(1,1)~=data(2,1))&&~((data(1,1)==0)&&(data(1,2)>0)&&(data(2,1)>0)) % if no dirac
        if (data(1,1)==0)&&(data(1,2)==0)&&(data(2,1)>0) % if implied dirac
            data = data(2:size(data,1),:);
        end
        [thr,pp,bb] = tab_check(data);
        
        if (round(y_av)>round((1-pp(1))*thr(1)*bb(1)+pp(1)*thr(1)))||(round(y_av)<round((1-pp(1))*thr(1)*bb(1)))
            error('Average income is not consistent with Pareto coefficients.')
        end
    else % if dirac
        if (data(1,1)==0)&&(data(1,2)>0)&&(data(2,1)>0) % if implied dirac
            data = [0 0 NaN NaN; data];
        end
        if data(1,1)>=data(3,1)
            error('Thresholds should be in ascending order.')
        elseif data(2,2)>=data(3,2)
            error('Percentiles should be in ascending order.')
        elseif ~(data(1,2)==0||isnan(data(1,2)))
            error('If there is a dirac at the bottom of the distribution, the first percentile should be 0.')
        end
        if isnan(data(1,2))
            data(1,2)=0;
        end
        
        dirac = [data(1,1) data(2,2)-data(1,2)];
        if (dirac(1)==0)||(isnan(data(2,3))&&isnan(data(2,4)))
        data = data(3:size(data,1),:);
        [thr,pp,bb] = tab_check(data);
        if (round(y_av)>round((1-pp(1))*thr(1)*bb(1)+(pp(1)-dirac(1,2))*thr(1)))||(round(y_av)<round((1-pp(1))*thr(1)*bb(1)))
            error('Dirac is not consistent with average income and Pareto coefficients.')
        end
        else
        data = data(2:size(data,1),:);
        [thr,pp,bb] = tab_check(data);
        if round(y_av)~=round((1-pp(1))*thr(1)*bb(1)+dirac(1)*dirac(2))
            error('Dirac is not consistent with average income and Pareto coefficients.')
        end
        end
    end
    
%% 3. The first threshold is negative
else
    if data(2,1)~=0
        error('If a part of the distribution is negative, the second threshold should be 0.')
    elseif (isnan(data(1,1))&&(isnan(data(1,4))||(data(1,4)>=0)))
        error('Either the first threshold or the first average value should be provided.')
    elseif (~isnan(data(1,2)))&&(data(1,2)~=0)
        error('If the first threshold is negative, the corresponding percentile should be 0 or should not be assigned.')
    elseif (data(1,3)<data(1,1))||(data(1,3)>=0)
        error('Mean value of the negative part is inconsistent.')
    elseif data(3,1)<0
        error('Thresholds should be in ascending order.')
    elseif data(2,2)>=data(3,2)
        error('Percentiles should be in ascending order.')
    elseif abs(data(1,4))>abs(data(1,1))/2
        disp('If a negative threshold T is provided and if the corresponding average value A is such that |A|>|T|/2, negative values may be generated below T.')
        data(1,1)=NaN;
    end
    neg = [data(1,1) data(2,2) data(1,4)];
    if data(3,1)==0
        if data(4,1)<=0
            error('Thresholds should be in ascending order.')
        elseif data(4,2)>=data(5,1)
            error('Percentiles should be in ascending order.')
        end
        dirac = [0 data(3,2)-data(2,2)];
        data = data(4:size(data,1),:);
    else data = data(3:size(data,1),:);
    end
    
	[thr,pp,bb] = tab_check(data);
    
	if min(isnan(dirac))
        if (round(y_av)>round((1-pp(1))*thr(1)*bb(1)+neg(1,2)*neg(1,3)+(pp(1)-neg(1,2))*thr(1)))||(round(y_av)<round((1-pp(1))*thr(1)*bb(1)+neg(1,2)*neg(1,3)))
            error('Average income is not consistent with Pareto coefficients.')
        end
    else
        if (round(y_av)>round((1-pp(1))*thr(1)*bb(1)+neg(1,2)*neg(1,3)+(pp(1)-neg(1,2)-dirac(1,2))*thr(1)))||(round(y_av)<round((1-pp(1))*thr(1)*bb(1)+neg(1,2)*neg(1,3)))
            error('Dirac is not consistent with average income and Pareto coefficients.')
        end
	end    
end



function [thr,pp,bb] = tab_check(tab)

	thr = tab(:,1); % thresholds
	pp  = tab(:,2); % percentiles
	bb  = tab(:,3); % inverted Pareto coefficients
	av  = tab(:,4); % average values within brackets
        
	y = (1-pp).*thr.*bb;
	y1 = [y(2:end); 0];
	pp1 = [pp(2:end); 1];
    format long
	mu = (y-y1)./(pp1-pp);
	
    if (thr(1)==0)&&((pp(1)~=0)&&(~isnan(pp(1))))
        error('If 0 is not a dirac and if there is no negative thresholds, please do not include it. If 0 a dirac, write it twice.')
    end
    if ~issorted(thr)||~isnumeric(thr)
        error('Thresholds should be numeric values in ascending order.')
	elseif ~(length(unique(thr))==length(thr))
        error('A dirac can only be at the bottom of the distribution.')
    end
    if ~issorted(pp)||~isnumeric(pp)
        error('Percentiles should be numeric numbers in ascending order.')
	elseif ~min(pp>=0)||~min(pp<1)
        error('Percentiles should be between 0 and 1.')
	elseif ~(length(unique(pp))==length(pp))
        error('A dirac can only be at the bottom of the distribution.')
    end
	if ~min(isnan(bb))
        if ~isnumeric(bb)||~min(bb(~isnan(bb))>1)
            error('Pareto coefficients should be numeric numbers greater than 1.')
        end
    end
    if ~min(isnan(mu))
        if ~min(mu(~isnan(mu))>thr(~isnan(mu)))||~min(max((thr(2:end)>mu(1:end-1)),isnan(mu(1:end-1))))
            error('Implied mean values within brackets are not consistent.')
        end
    end
	if ~min(isnan(av))
        if ~min(isnumeric(av))||~min(av(~isnan(av))>thr(~isnan(av)))||~min(max((thr(2:end)>av(1:end-1)),isnan(av(1:end-1))))
        	error('Given average values are not consistent.')
        end
    end
	if max(~min(~isnan(av),~isnan(mu)))
        if ~min(round(av(min(~isnan(av),~isnan(mu))))==round(mu(min(~isnan(av),~isnan(mu)))))
            error('Average values implied by Pareto coefficients and given average values are not consistent.')
        end
    end
	if max(min(isnan(av(:)),isnan(mu(:))))
        error('A Pareto coefficient or an average value should be provided for each bracket.')
    end
    
    av(isnan(av)) = mu(isnan(av));
    
	if thr(1)==0
        thr = thr(2:end);
         pp = pp(2:end);
         bb = bb(2:end);
         av = av(2:end);
         mu = mu(2:end);
    end
    
    if isnan(bb(end))
        bb(end) = av(end)/thr(end);
    end
    for i=(length(bb)-1):(-1):1
        if isnan(bb(i))
            bb(i) = ((1-pp(i+1))*thr(i+1)*bb(i+1)+(pp(i+1)-pp(i))*av(i))/((1-pp(i))*thr(i));
        end
    end

disp('All inputs are ok.')