

% permet d'additionner deux distributions y1 et y2
% supposons que l'input comprenne des tabulations de revenus pour deux pays 1 et 2
% l'output donne les gperc de y1, y2 et y=y1+y2, ainsi que la country share (i.e. le ratio (moyenne y1)/(moyenne y)) au sein de chaque gperc de y
% la diffrence avec les copulas est bien sr que les populations s'additionnent: la distribution y=y1+y2 porte sur la population totale runie des pays 1 et 2
% Idalement, j'aimerais aussi que le programme puisse marcher avec n pays. 

%% 1. Preliminaries
close all; clear all;

currentfolder=pwd;
% Path to load data
loadpath=[currentfolder,'/rawdata/twodimAdditive'];

Y =[];

%% 3. Defines g-percentiles
disp('Defining g-percentiles...');

gp1 = linspace(0,0.99,100);
gp2 = linspace(0.991,0.999,9);
gp3 = linspace(0.9991,0.9999,9);
gp4 = linspace(0.99991,0.99999,9);

gperc = [gp1,gp2,gp3,gp4]';
gperc100 = 100*gperc;

%% 2. Loads data
disp('Loading data...');

cd(loadpath);
dd = dir('*.xls*');

inputsNames = {dd.name};

n = numel(inputsNames);

Ntot=0;
for ii = 1:n
    [rawdata,txt] = xlsread(inputsNames{ii},1);
%   rawdata = readtable(inputsNames{ii});
     Ni = round(rawdata(2,2));
    Ntot = Ntot+Ni;
end

lambda = max(ceil(1000000/Ntot),1);

for ii = 1:n

    disp(inputsNames{ii})
    
    cd(loadpath);
    [rawdata,txt] = xlsread(inputsNames{ii},1);
%    rawdata = readtable(inputsNames{ii});
     Ni = lambda*round(rawdata(2,2));
    rawdata = rawdata(3:end,:);

    if size(rawdata,2)>=4
        if ~strcmp(txt(end,4),'yav')
            rawdata = rawdata(:,1:3);
        end
    elseif size(rawdata,2)>4
        rawdata = rawdata(:,1:4);
    end
    
    filename = strcat('gperc_',strtok(inputsNames(ii),'.'),'.csv');
  
codespath=[currentfolder,'/codes'];
cd(codespath);
try
    [raw.thr,raw.p,raw.b,y_av,dirac,neg] = inputs_check(rawdata);
catch
    continue
end

if (raw.p(1)==0)&&(raw.thr(1)>0)
    dirac = raw.thr(1);
    p0 = 0;
    raw.p = raw.p(2:end);
    raw.thr = raw.thr(2:end);
    raw.b = raw.b(2:end);
end

if ~(length(dirac)==1)
      p0  = dirac(2);
	dirac = dirac(1);
else
    dirac = 0;
      p0  = NaN;
end

if ~(length(neg)==1)
    thr_neg = neg(1);
	 p_neg  = neg(2);
	av_neg  = neg(3);
else
    thr_neg = 0;
	 p_neg  = 0;
	av_neg  = 0;
end

clear rawdata;
cd(currentfolder);


%% 4. Estimates the thresholds and the average incomes at g-percentiles
disp('Estimating the thresholds and the average incomes at g-percentiles...')

cd(codespath);
try
    [thr0,B_pos] = quantile(shift_p(gperc(gperc>=p_neg),p_neg),shift_thr(raw.thr,dirac),shift_p(raw.p,p_neg),shift_b(raw.thr,raw.b,dirac),shift_av(y_av,dirac,p_neg,av_neg),shift_p(p_neg+p0,p_neg));
catch
    continue
end
thr_pos = dirac+thr0;

B_pos = shift_b(thr_pos,B_pos,-dirac);
gperc_pos = gperc(gperc>=p_neg);

if p_neg>0
    try
        thr_neg = neg_part(gperc(gperc<p_neg)/p_neg,thr_neg,av_neg);
    catch
        continue
    end
else
    thr_neg = [];
end

thr = sort([thr_neg; thr_pos]);

%%

u = linspace(1/(Ni+1),1-1/(Ni+1),Ni)';

Yi = interp1(gperc,thr,u,'linear','extrap');
ind = ii*ones(Ni,1);
Y = [Y; Yi ind];

%% 5. Computes average incomes above g-percentiles
disp('Computing average incomes above g-percentiles...')

av = zeros(size(gperc));

if isnan(p0)
    p0 = 0;
end

av(gperc>p_neg+p0)= thr(gperc>p_neg+p0).*B_pos(gperc_pos>p0);

rk = find(gperc>p_neg+p0,1);
for i=1:(rk-1)
    av(i) = (integral(@(p)(interp1(gperc,thr,p)),gperc(i),gperc(rk))+(1-gperc(rk))*av(rk))./(1-gperc(i));
end

B = NaN(size(gperc));
B(gperc>=p_neg) = B_pos;

%% 6. Exports the output
disp('Exporting the output...')

table = [gperc100,thr,av,B];

outputpath=[currentfolder,'/output/twodimAdditive'];
cd(outputpath);

header = {'p','thr','ytop','b'};

fileID = fopen(filename{1}, 'w');
if fileID == -1; error('Cannot open file: %s', filename); end
fprintf(fileID, '%s,', header{1:end-1});
fprintf(fileID, '%s\n', header{end});
fprintf(fileID, '%.3f,%.4f,%.4f,%.4f\n', table');
fclose(fileID);

end

%% 7. 

Ys=Y(~isnan(Y(:,1)),:);
[~,I] = sort(Ys(:,1));
%Y = Y(I,:);
Y0=Ys(I,1);
ind=Ys(I,2);

Npop = size(Ys,1);
u = linspace(1/(Npop+1),1-1/(Npop+1),Npop)';

cd(codespath);
thr = interp1(u,Y0(:,1),gperc,'linear','extrap');
av_pop = flip(cumsum(flip(Y0(:,1)))./(1:length(Y0(:,1)))');
av = interp1(u,av_pop,gperc,'linear','extrap');
B = av./thr;

country = zeros(127,n);

gperc1=[gperc;1];


for i=1:127
    
    for j=1:n
        country(i,j)=sum(Y0((u>gperc1(i))&(u<=gperc1(i+1))&(ind==j)))/sum(Y0((u>gperc1(i))&(u<=gperc1(i+1))));
    end
    
end

%country(isnan(country)) = 0;

table = [gperc100,thr,av,B,country];

outputpath=[currentfolder,'/output/twodimAdditive'];
cd(outputpath);

header = [{'p','thr','ytop','b'} inputsNames];

fileID = fopen('twodimAdditive.csv', 'w');
if fileID == -1; error('Cannot open file: %s', filename); end
fprintf(fileID, '%s,', header{1:end-1});
fprintf(fileID, '%s\n', header{end});
fprintf(fileID, strcat('%.3f',repmat(',%.4f',1,n+3),'\n'), table');
fclose(fileID);
