%% Simulation of a population of taxpayers using income tax data
% 
% Npop=34546115 => about 1 hour
%
% Steps of the program:
%   * 1. Preliminaries
%   * 2. Loads data
%   * 3. Generates a vector of random numbers uniformly distributed on [0,1]
%   * 4. Simulates the population
%   * 5. Exports the population
%
%% 1. Preliminaries 
close all; clear all;
currentfolder=pwd;
% Path to load data
loadpath=[currentfolder,'/data'];

% Number of taxpayers to be generated
Npop = 100000;%34546115;

%% 2. Loads data
disp('Loads data');

cd(loadpath);
 [tabulation] = xlsread('taxdata_pchip','Tabulation'); %thresholds
  tab.thr = tabulation(:,1); %thresholds
  tab.p   = tabulation(:,2); %percentiles
  tab.b   = tabulation(:,3); %inverted Pareto coefficients
     T    = length(tab.thr); %number of thresholds in the tax tabulation

 [y_av] = xlsread('taxdata_pchip','Average income')'; %average income

clear tabulation;
cd(currentfolder);

%% 3. Generates a vector of random numbers uniformly distributed on [0,1]
disp('Generates a vector of random numbers uniformly distributed on [0,1]');

u = rand(Npop,1); %vector of Npop random numbers uniformly drawn on [0,1]
u = sort(u); %sorts the coefficients of u

%% 4. Simulates the population
disp('Simulates the population');

[sim_pop,p0] = quantile_trapz(tab.thr,tab.p,tab.b,y_av,u);

%% 5. Exports the simulated population
disp('Exports the simulated population');

dlmwrite('sim_pop_trapz.txt',sim_pop,'precision',9);

%% 6. 

%b_fin = flip(cumsum(flip(sim_pop)))./sim_pop
