/********Stata code upgrading urban survey tabulations on the basis of top income tax data*********/


#delimit;
clear;
set more off;

cd "D:\Dropbox\WIDChina\PYZ2016Simulations\";


/************importing upgrade factors*****/
import excel ChinaUrban_ParetoCurves_yf_raw.xlsx, sheet("UpgradeFactors") cellrange(A1:w26) firstrow clear;
replace p=1000*p;
sort p;
save StataFiles/temp/upgradefactors.dta, replace;

/********** upgrading urban gperc series using annual 2006-2010 upgrade factors****/
use StataFiles/gperc_yf_UrbanChina_raw.dta, clear;
sort p;
merge m:1 p using StataFiles/temp/upgradefactors.dta, nogen;
sort year p;
forvalues year=2006/2010 
{;
replace ythr=ythr*upythr`year' if year==`year' & p>=80000; 
replace yint=yint*upyint`year' if year==`year' & p>=80000; 
replace ytop=ytop*upyint`year' if year==`year' & p==99500; 
};
replace ythr=ythr*upythr if year>2010 & p>=80000; 
replace yint=yint*upyint if year>2010 & p>=80000; 
replace ytop=ytop*upyint if year>2010 & p==99500;
replace ythr=ythr*upythr if year<2006 & p>=80000; 
replace yint=yint*upyint if year<2006 & p>=80000; 
replace ytop=ytop*upyint if year<2006 & p==99500; 
gsort year -p;
replace ytop=(f*yint+(100000-p[_n-1])*ytop[_n-1])/(100000-p) if p<99500;
replace b=ytop/ythr;
replace ythr=. if p>99500;
replace yint=. if p>99500;
replace ytop=. if p>99500;
replace b=. if p>99500;
drop u*;
sort year p;
/***consistency check*/
/*gen check=0;
replace check=1 if yint<ythr | (yint>ythr[_n+1] & p<99999);
/*no consistency pb at all*/
ta check;
drop check;*/
save StataFiles/variants/yearlyupgrade/gperc_yf_UrbanChina_yearlyupgradev1.dta, replace;


/********** upgrading urban gperc series using 2011-2015 provincial data****/
use StataFiles/gperc_yf_UrbanChina_raw.dta, clear;
sort p;
merge m:1 p using StataFiles/temp/upgradefactors.dta, nogen;
sort year p;
forvalues year=2006/2010 
{;
replace ythr=ythr*upythr`year' if year==`year' & p>=80000; 
replace yint=yint*upyint`year' if year==`year' & p>=80000; 
replace ytop=ytop*upyint`year' if year==`year' & p==99500; 
};
forvalues year=2011/2015 
{;
replace ythr=ythr*upythr`year' if year==`year' & p>=80000; 
replace yint=yint*upyint`year' if year==`year' & p>=80000; 
replace ytop=ytop*upyint`year' if year==`year' & p==99000; 
};
replace ythr=ythr*upythr if year<2006 & p>=80000; 
replace yint=yint*upyint if year<2006 & p>=80000; 
replace ytop=ytop*upyint if year<2006 & p==99500; 
gsort year -p;
replace ytop=(f*yint+(100000-p[_n-1])*ytop[_n-1])/(100000-p) if p<99500 & year<=2010;
replace ytop=(f*yint+(100000-p[_n-1])*ytop[_n-1])/(100000-p) if p<99000 & year>2010;
replace b=ytop/ythr;
replace ythr=. if p>99500 & year<=2010;
replace yint=. if p>99500 & year<=2010;
replace ytop=. if p>99500 & year<=2010;
replace b=. if p>99500 & year<=2010;
replace ythr=. if p>99000 & year>2010;
replace yint=. if p>99000 & year>2010;
replace ytop=. if p>99000 & year>2010;
replace b=. if p>99000 & year>2010;
drop u*;
sort year p;
save StataFiles/variants/yearlyupgrade/gperc_yf_UrbanChina_yearlyupgradev2.dta, replace;

/***************extracting table 1y v1: summary statistics*/
use StataFiles/variants/yearlyupgrade/gperc_yf_UrbanChina_yearlyupgradev1.dta, clear;
sort year p;
gen p0=0 if p==0;
replace p0=p/1000+(yint/(ythr[_n+1]-ythr))*f/1000 if ythr==0 & ythr[_n+1]>0 & year[_n+1]==year;
keep if p==0 | p==10000 | p==20000 | p==30000 | p==40000 | p==50000 | p==60000 | p==70000 | p==80000 | p==90000 | p==99000 | p==99900 | p==99990 | p==99999;
sort year p; 
gen ymean=ytop if p==0;
gen bottom10=1-0.9*ytop[_n+1]/ytop if p==0;
gen bottom50=1-0.5*ytop[_n+5]/ytop if p==0;
gen top10=0.1*ytop[_n+9]/ytop if p==0;
gen middle40=1-top10-bottom50 if p==0;
gen top1=0.01*ytop[_n+10]/ytop if p==0;
gen top01=0.001*ytop[_n+11]/ytop if p==0;
gen top001=0.0001*ytop[_n+12]/ytop if p==0;
gen top0001=0.00001*ytop[_n+13]/ytop if p==0;
gen p10=ythr[_n+1]/ymean if p==0;
gen p50=ythr[_n+5]/ymean if p==0;
gen p90=ythr[_n+9]/ymean if p==0;
gen p99=ythr[_n+10]/ymean if p==0;
gen p999=ythr[_n+11]/ymean if p==0;
gen p9999=ythr[_n+12]/ymean if p==0;
gen p99999=ythr[_n+13]/ymean if p==0;
gen b10=ytop[_n+1]/ythr[_n+1] if p==0;
gen b50=ytop[_n+5]/ythr[_n+5] if p==0;
gen b90=ytop[_n+9]/ythr[_n+9] if p==0;
gen b99=ytop[_n+10]/ythr[_n+10] if p==0;
gen b999=ytop[_n+11]/ythr[_n+11] if p==0;
gen b9999=ytop[_n+12]/ythr[_n+12] if p==0;
gen b99999=ytop[_n+13]/ythr[_n+13] if p==0;
gen p20=ythr[_n+2]/ymean if p==0;
gen p30=ythr[_n+3]/ymean if p==0;
gen p40=ythr[_n+4]/ymean if p==0;
gen p60=ythr[_n+6]/ymean if p==0;
gen p70=ythr[_n+7]/ymean if p==0;
gen p80=ythr[_n+8]/ymean if p==0;
gen p1020=(0.9*ytop[_n+1]-0.8*ytop[_n+2])/(0.1*ymean);
gen p2030=(0.8*ytop[_n+2]-0.7*ytop[_n+3])/(0.1*ymean);
gen p3040=(0.7*ytop[_n+3]-0.6*ytop[_n+4])/(0.1*ymean);
gen p4050=(0.6*ytop[_n+4]-0.5*ytop[_n+5])/(0.1*ymean);
gen p5060=(0.5*ytop[_n+5]-0.4*ytop[_n+6])/(0.1*ymean);
gen p6070=(0.4*ytop[_n+6]-0.3*ytop[_n+7])/(0.1*ymean);
gen p7080=(0.3*ytop[_n+7]-0.2*ytop[_n+8])/(0.1*ymean);
gen p8090=(0.2*ytop[_n+8]-0.1*ytop[_n+9])/(0.1*ymean);
keep if p==0;
keep year ymean bottom10 bottom50 middle40 top10 top1 top01 top001 top0001 p0 p10 p50 p90 p99 p999 p9999 p99999 b10 b50 b90 b99 b999 b9999 b99999 p20 p30 p40 p60 p70 p80 p1020 p2030 p3040 p4050 p5060 p6070 p7080 p8090;
order year ymean bottom10 bottom50 middle40 top10 top1 top01 top001 top0001 p0 p10 p50 p90 p99 p999 p9999 p99999 b10 b50 b90 b99 b999 b9999 b99999 p20 p30 p40 p60 p70 p80 p1020 p2030 p3040 p4050 p5060 p6070 p7080 p8090;
save StataFiles/variants/yearlyupgrade/table1yfu_yearlyupgradev1.dta, replace;
export excel using StataFiles/variants/yearlyupgrade/table1yfu_yearlyupgradev1.xlsx, firstrow(variables) replace;

/***************extracting table 1y v2: summary statistics*/
use StataFiles/variants/yearlyupgrade/gperc_yf_UrbanChina_yearlyupgradev2.dta, clear;
sort year p;
gen p0=0 if p==0;
replace p0=p/1000+(yint/(ythr[_n+1]-ythr))*f/1000 if ythr==0 & ythr[_n+1]>0 & year[_n+1]==year;
keep if p==0 | p==10000 | p==20000 | p==30000 | p==40000 | p==50000 | p==60000 | p==70000 | p==80000 | p==90000 | p==99000 | p==99900 | p==99990 | p==99999;
sort year p; 
gen ymean=ytop if p==0;
gen bottom10=1-0.9*ytop[_n+1]/ytop if p==0;
gen bottom50=1-0.5*ytop[_n+5]/ytop if p==0;
gen top10=0.1*ytop[_n+9]/ytop if p==0;
gen middle40=1-top10-bottom50 if p==0;
gen top1=0.01*ytop[_n+10]/ytop if p==0;
gen top01=0.001*ytop[_n+11]/ytop if p==0;
gen top001=0.0001*ytop[_n+12]/ytop if p==0;
gen top0001=0.00001*ytop[_n+13]/ytop if p==0;
gen p10=ythr[_n+1]/ymean if p==0;
gen p50=ythr[_n+5]/ymean if p==0;
gen p90=ythr[_n+9]/ymean if p==0;
gen p99=ythr[_n+10]/ymean if p==0;
gen p999=ythr[_n+11]/ymean if p==0;
gen p9999=ythr[_n+12]/ymean if p==0;
gen p99999=ythr[_n+13]/ymean if p==0;
gen b10=ytop[_n+1]/ythr[_n+1] if p==0;
gen b50=ytop[_n+5]/ythr[_n+5] if p==0;
gen b90=ytop[_n+9]/ythr[_n+9] if p==0;
gen b99=ytop[_n+10]/ythr[_n+10] if p==0;
gen b999=ytop[_n+11]/ythr[_n+11] if p==0;
gen b9999=ytop[_n+12]/ythr[_n+12] if p==0;
gen b99999=ytop[_n+13]/ythr[_n+13] if p==0;
gen p20=ythr[_n+2]/ymean if p==0;
gen p30=ythr[_n+3]/ymean if p==0;
gen p40=ythr[_n+4]/ymean if p==0;
gen p60=ythr[_n+6]/ymean if p==0;
gen p70=ythr[_n+7]/ymean if p==0;
gen p80=ythr[_n+8]/ymean if p==0;
gen p1020=(0.9*ytop[_n+1]-0.8*ytop[_n+2])/(0.1*ymean);
gen p2030=(0.8*ytop[_n+2]-0.7*ytop[_n+3])/(0.1*ymean);
gen p3040=(0.7*ytop[_n+3]-0.6*ytop[_n+4])/(0.1*ymean);
gen p4050=(0.6*ytop[_n+4]-0.5*ytop[_n+5])/(0.1*ymean);
gen p5060=(0.5*ytop[_n+5]-0.4*ytop[_n+6])/(0.1*ymean);
gen p6070=(0.4*ytop[_n+6]-0.3*ytop[_n+7])/(0.1*ymean);
gen p7080=(0.3*ytop[_n+7]-0.2*ytop[_n+8])/(0.1*ymean);
gen p8090=(0.2*ytop[_n+8]-0.1*ytop[_n+9])/(0.1*ymean);
keep if p==0;
keep year ymean bottom10 bottom50 middle40 top10 top1 top01 top001 top0001 p0 p10 p50 p90 p99 p999 p9999 p99999 b10 b50 b90 b99 b999 b9999 b99999 p20 p30 p40 p60 p70 p80 p1020 p2030 p3040 p4050 p5060 p6070 p7080 p8090;
order year ymean bottom10 bottom50 middle40 top10 top1 top01 top001 top0001 p0 p10 p50 p90 p99 p999 p9999 p99999 b10 b50 b90 b99 b999 b9999 b99999 p20 p30 p40 p60 p70 p80 p1020 p2030 p3040 p4050 p5060 p6070 p7080 p8090;
save StataFiles/variants/yearlyupgrade/table1yfu_yearlyupgradev2.dta, replace;
export excel using StataFiles/variants/yearlyupgrade/table1yfu_yearlyupgradev2.xlsx, firstrow(variables) replace;
